;;; build.el --- Builds EMchat.
;;
;; Copyright (C) 2006 Sebastian Freundt
;; Copyright (C) 2007 - 2011 Steve Youngs
;;
;; Author: Sebastian Freundt <hroptatyr@sxemacs.org>
;; Maintainer: EMchat Development Team
;; Keywords: internal
;;
;; This file is part of EMchat.

;; Redistribution and use in source and binary forms, with or without
;; modification, are permitted provided that the following conditions
;; are met:
;;
;; 1. Redistributions of source code must retain the above copyright
;;    notice, this list of conditions and the following disclaimer.
;;
;; 2. Redistributions in binary form must reproduce the above copyright
;;    notice, this list of conditions and the following disclaimer in the
;;    documentation and/or other materials provided with the distribution.
;;
;; 3. Neither the name of the author nor the names of any contributors
;;    may be used to endorse or promote products derived from this
;;    software without specific prior written permission.
;;
;; THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
;; IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;; DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
;; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
;; SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
;; BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
;; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
;; OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
;; IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;;
;;; Synched up with: Not in FSF.

;;; Commentary:
;;
;;  This file is only used during EMchat builds, it does what the name
;;  implies... builds the lisp, plus it generates auto-autoloads.el,
;;  custom-load.el and custom-define.el files.

;;; Code:
(setq stack-trace-on-error t)

(defvar wd default-directory)
(push wd load-path)

(defvar needed '(cus-dep autoload bytecomp byte-optimize font))
(mapcar #'require needed)

(defalias #'executable-find #'ignore)

(let ((pname "emchat")
      (adir wd))
  (update-autoload-files adir pname)
  (Custom-make-dependencies adir)
  (update-custom-define-files adir pname))

(load "prepsrc")

(mapcar
 #'(lambda (sym)
     (let ((file (symbol-name sym)))
       (when (string= file "custom-defines.el")
	 (autoload 'emchat-emphasis-custom-with-format
	   "emchat-emphasis" nil nil 'macro))
       (byte-compile-file (expand-file-name file wd))))
 srcfiles)

;; indicate success
(kill-emacs 0)

;;; build.el ends here
