;; emchat-setup.el --- Setup user directory and files for EMchat.

;; Copyright (C) 2002 - 2011 Steve Youngs

;; Author:        Steve Youngs <steve@emchat.org>
;; Maintainer:    Steve Youngs <steve@emchat.org>
;; Created:       2002-10-03
;; Homepage:      http://www.emchat.org/
;; Keywords:      comm ICQ

;; This file is part of EMchat.

;; Redistribution and use in source and binary forms, with or without
;; modification, are permitted provided that the following conditions
;; are met:
;;
;; 1. Redistributions of source code must retain the above copyright
;;    notice, this list of conditions and the following disclaimer.
;;
;; 2. Redistributions in binary form must reproduce the above copyright
;;    notice, this list of conditions and the following disclaimer in the
;;    documentation and/or other materials provided with the distribution.
;;
;; 3. Neither the name of the author nor the names of any contributors
;;    may be used to endorse or promote products derived from this
;;    software without specific prior written permission.
;;
;; THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
;; IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
;; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;; DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
;; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
;; SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
;; BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
;; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
;; OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
;; IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

(defconst emchat-world-user-file-template
  "
Literally you can put anything you like in this resource file, such as
this introduction.  EMchat will only consider a line to be a valid buddy
if...

    -- `:icq' starts at column zero (there's no leading whitespace or
       characters).

    -- There is one space between `:icq' and a UIN.

    -- There is one space between a UIN and a buddy name.

    -- The buddy name does NOT contain any colon `:' characters.
       Buddy names can contain whitespace anywhere except as the
       first character.

    -- There is one space between the buddy name and any group names.
       Group names begin with a colon `:'.

So, for example...

:icq 34307457 emchat
:icq 12345678 me
:icq 88888888 the queen :royalty

This way, it defines three buddies: \"emchat\", \"me\", and \"the
queen\". It reads alias name until the end of the line, or until the
first group name.  In that example, the buddy \"the queen\" is in
the group `:royalty'.

Adding your own UIN:

Just change \"12345678 me\" above to your UIN/alias.  And don't
forget to change the (setq emchat-user-alias \"me\") line in your
`user-init-file' to match.

BTW, that 1st buddy up there, 34307457 emchat, is a valid UIN, it's
mine. :-)

Remember to M-x emchat-world-update after changing this rc file.

"
  "Template used to create user's initial EMchat rc file.")

(eval-when-compile
  (defvar emchat-history-directory))

;;;###autoload
(defun emchat-setup ()
  "Setup your personal EMchat directory.

This directory, which defaults to '~/.emchat/', holds the resource file
for your ICQ contacts and any log files.  It's also a good place to
put your sound files."
  (interactive)
  (unless (featurep '(and emchat emchat-history emchat-world emchat-setup))
    (require 'emchat)
    (require 'emchat-history)
    (require 'emchat-world)
    (require 'emchat-setup))
  ;; Create necessary directories.
  (unless (file-directory-p emchat-directory)
    (make-directory emchat-directory t))
  (unless (file-directory-p emchat-sound-directory)
    (make-directory emchat-sound-directory t))
  (unless (file-directory-p emchat-history-directory)
    (make-directory emchat-history-directory t))
  ;; Create a world file if needed.
  (unless (file-exists-p emchat-world-rc-filename)
    (save-excursion
      (switch-to-buffer (find-file emchat-world-rc-filename))
      (insert (symbol-value 'emchat-world-user-file-template))
      (save-buffer)
      (kill-buffer nil)))
  (message "Don't forget to edit %s to your requirements"
	   emchat-world-rc-filename))

(provide 'emchat-setup)
;;; emchat-setup.el ends here
